# Created on Tue Dec 23 21:15:19 2014

# Author: XiaoTao Wang
# Organization: HuaZhong Agricultural University

import logging, os, subprocess
import numpy as np
from mirnylib.h5dict import h5dict

log = logging.getLogger(__name__)

def initialize(dataFolder, genomeName, gapName, chroms, template):
    ## Necessary Modules
    from runHiC.chiclib import myGenome
    
    dataLocation = os.path.abspath(os.path.expanduser(dataFolder))
    genomeFolder = os.path.join(dataLocation, genomeName)
    ## Generate a dummy gap file under genome folder if there's no one yet
    gapFile = os.path.join(genomeFolder, gapName)
    if not os.path.exists(gapFile):
        tmpfil = open(gapFile, 'wb')
        tmpfil.write('0\tNA1000\t0\t0\t0\tN\t0\tcentromere\tno\n')
        tmpfil.flush()
        tmpfil.close()
    
    # Python Genome Object
    genome_db = myGenome(genomeFolder, readChrms = chroms,
                         chrmFileTemplate = template, gapFile = gapName)
    
    return dataLocation, genomeFolder, genome_db

def calculateStep(length, minlen, approxStep=10, maxSteps=4):
    """
    Returns minimum length and step based on the length of sequence and
    proposed minimum length.
    """
    actualDif = length - minlen
    if actualDif < approxStep * 0.6:
        return length, 100
    
    numIter = np.array(np.around(actualDif / float(approxStep)), dtype=int)
    if numIter == 0:
        numIter = 1
    if numIter > maxSteps:
        numIter = maxSteps
    actualStep = actualDif / numIter
    
    minlen = length - actualStep * numIter
    
    return minlen, actualStep

def juncSeqCountFASTQ(Fastq_1, Fastq_2, enzyme):
    """
    Scan each read to match so-called junction sequence.
    
    A paucity of ligation junctions in a Hi-C library suggests that the
    ligation failed.
    
    """
    import Bio.Restriction
    
    enzyme_site = eval('Bio.Restriction.%s.site' % enzyme)
    cutsite = eval('Bio.Restriction.%s.charac' % enzyme)[:2]
    
    Dict = {'A':'T', 'C':'G', 'G':'C', 'T':'A'}
    reverse = ''.join([Dict[i] for i in enzyme_site])
    
    if cutsite[-1]:
        jplus = enzyme_site[:cutsite[-1]] + enzyme_site[cutsite[0]:]
        jminus = reverse[:cutsite[-1]] + reverse[cutsite[0]:]
        jminus = jminus[::-1]
    else:
        jplus = enzyme_site[:None] + enzyme_site[cutsite[0]:]
        jminus = reverse[:None] + reverse[cutsite[0]:]
        jminus = jminus[::-1]
    
    if Fastq_1.endswith('.fastq.gz'):
        pread_1 = subprocess.Popen(['gunzip', Fastq_1, '-c'],
                                    stdout = subprocess.PIPE, bufsize = -1)
    else:
        pread_1 = subprocess.Popen(['cat', Fastq_1],
                                    stdout = subprocess.PIPE, bufsize = -1)
    
    if Fastq_2.endswith('.fastq.gz'):
        pread_2 = subprocess.Popen(['gunzip', Fastq_2, '-c'],
                                    stdout = subprocess.PIPE, bufsize = -1)
    else:
        pread_2 = subprocess.Popen(['cat', Fastq_2],
                                    stdout = subprocess.PIPE, bufsize = -1)
                                  
    inStream_1 = pread_1.stdout
    inStream_2 = pread_2.stdout
    
    Len = 0
    Count = 0
    
    if jplus == jminus:
        while True:
            line = inStream_1.readline()
            
            try:
                assert line[0] == '@'
            except IndexError:
                break
            
            Block_1 = [line, inStream_1.readline(), inStream_1.readline(),
                       inStream_1.readline()]
            
            Block_2 = [inStream_2.readline(), inStream_2.readline(),
                       inStream_2.readline(), inStream_2.readline()]
            
            check_1 = (jplus in Block_1[1])
            check_2 = (jplus in Block_2[1])
            
            Len += 1
            if check_1 or check_2:
                Count += 1
    else:
        while True:
            line = inStream_1.readline()
            
            try:
                assert line[0] == '@'
            except IndexError:
                break
            
            Block_1 = [line, inStream_1.readline(), inStream_1.readline(),
                       inStream_1.readline()]
            
            Block_2 = [inStream_2.readline(), inStream_2.readline(),
                       inStream_2.readline(), inStream_2.readline()]
            
            check_1 = (jplus in Block_1[1]) or (jminus in Block_1[1])
            check_2 = (jplus in Block_2[1]) or (jminus in Block_2[1])
            
            Len += 1
            if check_1 or check_2:
                Count += 1
    pread_1.communicate()
    pread_2.communicate()
    
    return Len, Count

def commandExists(command):
    "Check if the bash command exists"
    command = command.split()[0]
    if subprocess.call(['which', command]) != 0:
        return False
    return True

def gzipWriter(filename):
    """
    Create a writing process with gzip or parallel gzip (pigz) attached to
    it.
    
    """
    filename = os.path.abspath(filename)
    
    with open(filename, 'wb') as outFile:
        if commandExists("pigz"):
            writer = ["pigz", "-c", "-4"]
        else:
            writer = ["gzip", "-c", "-1"]

        pwrite = subprocess.Popen(writer, stdin = subprocess.PIPE,
                                  stdout = outFile, shell = False,
                                  bufsize = -1)
    return pwrite

def uncompressSRA(sra, fastqF, bamF, hdf5F):
    
    if not commandExists('fastq-dump'):
        raise ValueError('Please install fastq-dump first!')
    
    pread = subprocess.Popen(['fastq-dump', sra, "-Z", "--split-files"],
                              stdout = subprocess.PIPE, bufsize = -1)
    
    inStream = pread.stdout
    
    outname = os.path.split(sra)[1].replace('.sra', '') + '_{0}.fastq.gz'
    outf1 = os.path.join(fastqF,outname).format(1)
    outf2 = os.path.join(fastqF,outname).format(2)
    outb1 = os.path.join(bamF,os.path.split(outf1)[1].replace('.fastq.gz','.bam'))
    outb2 = os.path.join(bamF,os.path.split(outf2)[1].replace('.fastq.gz','.bam'))
    hdf5 = os.path.join(hdf5F,os.path.split(sra)[1].replace('.sra','.hdf5'))
    outProc1 = gzipWriter(outf1)
    outProc2 = gzipWriter(outf2)
    outStream1 = outProc1.stdin
    outStream2 = outProc2.stdin
    
    count = 0
    while True:
        line = inStream.readline()
        
        try:
            assert line[0] == '@'
        except AssertionError:
            raise IOError('Invalid fastq file')
        except IndexError:
            break
        
        fastq_entry_1 = (line, inStream.readline(),
                         inStream.readline(), inStream.readline())
        outStream1.writelines(fastq_entry_1)
        
        fastq_entry_2 = (inStream.readline(), inStream.readline(),
                         inStream.readline(), inStream.readline())
        outStream2.writelines(fastq_entry_2)
        
        count += 1
    
    outProc1.communicate()
    outProc2.communicate()
    
    return [(outf1, outb1, outf2, outb2, hdf5)]

def linkRawFASTQ(fq1, fq2, chunkF, bamF, hdf5F):
    
    nfq1 = os.path.join(chunkF, os.path.split(fq1)[1])
    nfq2 = os.path.join(chunkF, os.path.split(fq2)[1])
    os.symlink(fq1, nfq1)
    os.symlink(fq2, nfq2)
    if fq1.endswith('.fastq'):
        outb1 = os.path.join(bamF,os.path.split(fq1)[1].replace('.fastq','.bam'))
        hdf5 = os.path.join(hdf5F,os.path.split(fq1)[1].replace('_1.fastq','.hdf5'))
    else:
        outb1 = os.path.join(bamF,os.path.split(fq1)[1].replace('.fastq.gz','.bam'))
        hdf5 = os.path.join(hdf5F,os.path.split(fq1)[1].replace('_1.fastq.gz','.hdf5'))
    if fq2.endswith('.fastq'):
        outb2 = os.path.join(bamF,os.path.split(fq2)[1].replace('.fastq','.bam'))
    else:
        outb2 = os.path.join(bamF,os.path.split(fq2)[1].replace('.fastq.gz','.bam'))
    
    return [(nfq1, outb1, nfq2, outb2, hdf5)]

def splitSRA(sra, chunkF, bamF, hdf5F, splitBy=4000000):
    
    if not commandExists('fastq-dump'):
        raise ValueError('Please install fastq-dump first!')

    outname = os.path.split(sra)[1].replace('.sra', '') + '_chunk{0}_{1}.fastq.gz'
    pread = subprocess.Popen(['fastq-dump', sra, "-Z", "--split-files"],
                              stdout = subprocess.PIPE, bufsize = -1)
    inStream = pread.stdout

    halted = False
    intermediates = []
    for counter in xrange(100000):
        outf1 = os.path.join(chunkF,outname).format(counter,1)
        outf2 = os.path.join(chunkF,outname).format(counter,2)
        outb1 = os.path.join(bamF,os.path.split(outf1)[1].replace('.fastq.gz','.bam'))
        outb2 = os.path.join(bamF,os.path.split(outf2)[1].replace('.fastq.gz','.bam'))
        hdf5 = os.path.join(hdf5F,os.path.split(outf1)[1].replace('_1.fastq.gz','.hdf5'))
        outProc1 = gzipWriter(outf1)
        outProc2 = gzipWriter(outf2)
        outStream1 = outProc1.stdin
        outStream2 = outProc2.stdin

        for j in xrange(splitBy):

            line = inStream.readline()

            try:
                assert line[0] == '@'
            except AssertionError:
                raise IOError('Invalid fastq file')
            except IndexError:
                halted = True
                break

            fastq_entry = (line, inStream.readline(),
                           inStream.readline(), inStream.readline())

            outStream1.writelines(fastq_entry)
            outStream2.writelines((inStream.readline(), inStream.readline(),
                       inStream.readline(), inStream.readline()))

        outProc1.communicate()
        outProc2.communicate()
        
        intermediates.append((outf1, outb1, outf2, outb2, hdf5))
        
        if halted:
            break
    
    return intermediates

def splitFASTQ(fq1, fq2, chunkF, bamF, hdf5F, splitBy=4000000):
    
    p1 = os.path.split(fq1)[1].split('.')[0].split('_')
    outname1 = '_'.join(p1[:-1]) + '_chunk{0}_{1}.fastq.gz'
    p2 = os.path.split(fq2)[1].split('.')[0].split('_')
    outname2 = '_'.join(p2[:-1]) + '_chunk{0}_{1}.fastq.gz'
    outname = '_'.join(p2[:-1]) + '_chunk{0}.hdf5'
    
    if fq1.endswith('.fastq.gz'):
        pread_1 = subprocess.Popen(['gunzip', fq1, '-c'],
                                    stdout = subprocess.PIPE, bufsize = -1)
    else:
        pread_1 = subprocess.Popen(['cat', fq1],
                                    stdout = subprocess.PIPE, bufsize = -1)
    if fq2.endswith('.fastq.gz'):
        pread_2 = subprocess.Popen(['gunzip', fq2, '-c'],
                                    stdout = subprocess.PIPE, bufsize = -1)
    else:
        pread_2 = subprocess.Popen(['cat', fq2],
                                    stdout = subprocess.PIPE, bufsize = -1)
                                  
    inStream_1 = pread_1.stdout
    inStream_2 = pread_2.stdout
    
    halted = False
    intermediates = []
    for counter in xrange(100000):
        outf1 = os.path.join(chunkF,outname1).format(counter,p1[-1])
        outb1 = os.path.join(bamF,os.path.split(outf1)[1].replace('.fastq.gz','.bam'))
        outf2 = os.path.join(chunkF,outname2).format(counter,p2[-1])
        outb2 = os.path.join(bamF,os.path.split(outf2)[1].replace('.fastq.gz','.bam'))
        hdf5 = os.path.join(hdf5F,outname).format(counter)
        outProc1 = gzipWriter(outf1)
        outProc2 = gzipWriter(outf2)
        outStream1 = outProc1.stdin
        outStream2 = outProc2.stdin

        for j in xrange(splitBy):

            line = inStream_1.readline()

            try:
                assert line[0] == '@'
            except AssertionError:
                raise IOError('Invalid fastq file')
            except IndexError:
                halted = True
                break


            fastq_entry_1 = (line, inStream_1.readline(), inStream_1.readline(),
                             inStream_1.readline())
            fastq_entry_2 = (inStream_2.readline(), inStream_2.readline(),
                             inStream_2.readline(), inStream_2.readline())

            outStream1.writelines(fastq_entry_1)
            outStream2.writelines(fastq_entry_2)

        outProc1.communicate()
        outProc2.communicate()
        
        intermediates.append((outf1, outb1, outf2, outb2, hdf5))
        
        if halted:
            break
    
    return intermediates

def genchunks(arg_pool):
        
    chunkF, subbamF, subhdf5F, query, Format, database, chunkSize = arg_pool
        
    fastqDir = os.path.split(chunkF)[0]
    if Format == 'sra':
        sra = os.path.join(fastqDir, query + '.sra')
        if not os.path.exists(sra):
            return
        if not chunkSize:
            queue = uncompressSRA(sra, chunkF, subbamF, subhdf5F)
        else:
            queue = splitSRA(sra, chunkF, subbamF, subhdf5F, splitBy=chunkSize)
    else:
        try_1 = os.path.join(fastqDir, query + '_1.fastq')
        try_2 = os.path.join(fastqDir, query + '_1.fastq.gz')
        try_3 = os.path.join(fastqDir, query + '_2.fastq')
        try_4 = os.path.join(fastqDir, query + '_2.fastq.gz')
        if os.path.exists(try_1) and os.path.exists(try_3):
            Fq_1 = try_1
            Fq_2 = try_3
        elif os.path.exists(try_1) and os.path.exists(try_4):
            Fq_1 = try_1
            Fq_2 = try_4
        elif os.path.exists(try_2) and os.path.exists(try_3):
            Fq_1 = try_2
            Fq_2 = try_3
        elif os.path.exists(try_2) and os.path.exists(try_4):
            Fq_1 = try_2
            Fq_2 = try_4
        else:
            return
        if not chunkSize:
            queue = linkRawFASTQ(Fq_1, Fq_2, chunkF, subbamF, subhdf5F)
        else:
            queue = splitFASTQ(Fq_1, Fq_2, chunkF, subbamF, subhdf5F,
                               splitBy=chunkSize)
    pool = []
    for pre in queue:
        values = pre + (database[query],)
        pool.append(values)
    return pool
    
# Convert Matrix to Scipy Sparse Matrix
def toSparse(source, csr = False):
    """
    Convert intra-chromosomal contact matrices to sparse ones.
    
    Parameters
    ----------
    source : str
         Hdf5 file name.
    
    idx2label : dict
        A dictionary for conversion between zero-based indices and
        string chromosome labels.
    
    csr : bool
        Whether to use CSR (Compressed Row Storage) format or not.
    
    """
    import zipfile, tempfile
    from numpy.lib.format import write_array
    from scipy import sparse
    
    lib = h5dict(source, mode = 'r')
    
    ## Uniform numpy-structured-array format
    itype = np.dtype({'names':['bin1', 'bin2', 'IF'],
                          'formats':[np.int, np.int, np.float]})
    
    ## Create a Zip file in NPZ case
    if not csr:
        output = source.replace('.hm', '-sparse.npz')
    else:
        output = source.replace('.hm', '-csrsparse.npz')
    
    Zip = zipfile.ZipFile(output, mode = 'w', allowZip64 = True)
    fd, tmpfile = tempfile.mkstemp(suffix = '-numpy.npy')
    os.close(fd)
    
    log.log(21, 'Sparse Matrices will be saved to %s', output)
    log.log(21, 'Only intra-chromosomal matrices will be taken into account')
    log.log(21, 'Coverting ...')
    
    count = 0
    
    for i in lib:
        if (i != 'resolution') and (i != 'genomeInformation') and (len(set(i.split())) == 1):
            # Used for the dict-like key
            key = lib['genomeInformation']['idx2label'][int(i.split()[0])]
            
            log.log(21, 'Chromosome %s ...', key)
            # 2D-Matrix
            H = lib[i]
            
            if not csr:
                # Triangle Array
                Triu = np.triu(H)
                # Sparse Matrix in Memory
                x, y = np.nonzero(Triu)
                values = Triu[x, y]
                temp = np.zeros(values.size, dtype = itype)
                temp['bin1'] = x
                temp['bin2'] = y
                temp['IF'] = values
            else:
                temp = sparse.triu(H, format = 'csr')
            
            fname = key + '.npy'
            fid = open(tmpfile, 'wb')
            try:
                write_array(fid, np.asanyarray(temp))
                fid.close()
                fid = None
                Zip.write(tmpfile, arcname = fname)
            finally:
                if fid:
                    fid.close()
                    
            log.log(21, 'Done!')
            
            count += 1
    
    if count == 0:
        log.warning('Empty source file!')
            
    # Other information
    for i in ['resolution', 'genomeInformation']:
        fname = '.'.join([i, 'npy'])
        fid = open(tmpfile, 'wb')
        try:
            write_array(fid, np.asanyarray(lib[i]))
            fid.close()
            fid = None
            Zip.write(tmpfile, arcname = fname)
        finally:
            if fid:
                fid.close()
    
    os.remove(tmpfile)
    
    Zip.close()


# -*- coding: utf-8 -*-
"""
Created on Thu Jun 15 16:32:51 2017

@author: wxt
"""

import os, time, pp, random, subprocess, logging, shlex
import multiprocessing as mp
from collections import Counter

log = logging.getLogger(__name__)

def mpPool(per_worker, maximum_worker):
    
    local = os.environ["HOSTNAME"]
    ncpus = mp.cpu_count()
    n_worker = ncpus // per_worker
    if not n_worker:
        n_worker = 1
    n_worker = min(n_worker, maximum_worker)
    log.log(21, 'Launch %d processes on local compute node: %s',
            n_worker, local)
    
    return mp.Pool(n_worker), n_worker
        

class ppLocal(pp.Server):
    
    def __init__(self, per_worker, maximum_worker):
        
        local = os.environ["HOSTNAME"]
        ncpus = mp.cpu_count()
        n_worker = ncpus // per_worker
        if not n_worker:
            n_worker = 1
        self.n_worker = min(n_worker, maximum_worker)
        log.log(21, 'Launch %d processes on local compute node: %s',
                self.n_worker, local)
        pp.Server.__init__(self, ncpus=self.n_worker)
        

class ppServer(pp.Server):
    """
    Only support PBS-based clusters.
    """
    def __init__(self, per_worker, maximum_worker, port=60000):
        self.per_worker = per_worker
        self.maximum_worker = maximum_worker
        secret = self._genSecret()
        timeout = self._walltime_to_seconds()
        self._get_nodes()
        #local_worker = self._local_node()
        self.launch_server(port, secret, timeout)
        servers = self._collect_servers(port)
        pp.Server.__init__(self, ncpus=0, ppservers=servers, secret=secret,
                           socket_timeout=timeout)
    
    def _cal_worker(self, ncpus):
        
        n_worker = ncpus // self.per_worker
        if not n_worker:
            n_worker = 1
        n_worker = min(n_worker, self.maximum_worker)
        
        return n_worker
    
    def _local_node(self):
        
        local = os.environ["HOSTNAME"]
        local_ncpus = self.nodes[local]
        self.nodes.pop(local)
        n_worker = self._cal_worker(local_ncpus)
        
        log.log(21, 'Launch %d processes on local compute node: %s',
                n_worker, local)
        
        return n_worker
        
    def _get_nodes(self):
        pbs_nodefile = os.environ["PBS_NODEFILE"]
        procs = [line.rstrip() for line in open(pbs_nodefile,'r')]
        self.nodes = Counter(procs)
    
    def _collect_servers(self, port):
        
        servers = ()
        for node in self.nodes:
            tmp = '{0}:{1}'.format(node, port)
            servers += (tmp,)
        return servers
    
    def launch_server(self, port, secret, timeout):
        
        template = 'pbsdsh -h {0} ppserver.py -p {1} -w {2} -s {3} -t {4} -k {5}'
        for node in self.nodes:
            ncpus = self.nodes[node]
            n_worker = self._cal_worker(ncpus)
            command = template.format(node, port, n_worker, secret, 36000, timeout)
            command_list = shlex.split(command)
            pid = subprocess.Popen(command_list).pid
            log.log(21, 'Launch %d processes on remote compute node: %s',
                    n_worker, node)
            time.sleep(10)
    
    def _genSecret(self):
        
        tl = list(time.strftime('%Y%m%d%H%M%S',time.localtime(time.time())))
        random.shuffle(tl)
        secret = ''.join(tl)

        return secret
    
    def _walltime_to_seconds(self):
        
        walltime = os.environ['PBS_WALLTIME']
        if ':' in walltime:
            p = walltime.split(':')
            seconds = 3600*int(p[0]) + 60*int(p[1]) + int(p[2])
        else:
            seconds = int(walltime)
        
        return seconds


# Created on Mon May 25 16:57:52 2015

# Author: XiaoTao Wang
# Organization: HuaZhong Agricultural University

import logging, os, time, re, tempfile, gc
from textwrap import dedent
import numexpr, matplotlib
matplotlib.use('Agg')
import numpy as np
from mirnylib.genome import Genome
from hiclib.fragmentHiC import HiCdataset
from hiclib.hicShared import mydtype, mydtypeSorter, searchsorted, h5dictBinarySearch
from hiclib.binnedData import binnedData
from mirnylib import numutils
from mirnylib.numutils import uniqueIndex, fillDiagonal, externalMergeSort, completeIC
from mirnylib.h5dict import h5dict

## Plot Settings
import matplotlib.pyplot as plt
# Matplotlib Settings
matplotlib.rcParams['xtick.direction'] = 'out'
matplotlib.rcParams['ytick.direction'] = 'out'
matplotlib.rcParams['axes.labelsize'] = 13
matplotlib.rcParams['xtick.labelsize'] = 13
matplotlib.rcParams['ytick.labelsize'] = 13
matplotlib.rcParams['xtick.major.size'] = 8
matplotlib.rcParams['ytick.major.size'] = 8
matplotlib.rcParams['xtick.minor.size'] = 5
matplotlib.rcParams['ytick.minor.size'] = 5
matplotlib.rcParams['xtick.major.pad'] = 6
matplotlib.rcParams['ytick.major.pad'] = 6
matplotlib.rcParams['xtick.minor.pad'] = 6
matplotlib.rcParams['ytick.minor.pad'] = 6

colorPool = ['#A6CEE3', '#1F78B4', '#B2DF8A', '#33A02C', '#348ABD', '#A60628']

log = logging.getLogger(__name__)

class myGenome(Genome):
    
    def _extractChrmLabel(self, fastaName):
        # First assume a whole filename as input (e.g. 'chr01.fa')
        _, fastaName = os.path.split(fastaName)
        regexp = self.chrmFileTemplate % ('(.*)')
        search_results = re.search(regexp, fastaName)
        # If not, assume that only the name is supplied as input (e.g. 'chr01')
        if search_results is None:
            regexp = self.chrmFileTemplate.split('.')[0] % ('(.*)')
            search_results = re.search(regexp, fastaName)

        if search_results is None:
            raise Exception(
                'The filename {} does not match the template {}.'.format(
                fastaName, self.chrmFileTemplate))

        chrm_label = search_results.group(1)

        return chrm_label

# A Customized HiCdataset Class
class cHiCdataset(HiCdataset):
    
    def __init__(self, filename, genome, maximumMoleculeLength=500,
                 mode="a", tmpFolder="/tmp", dictToStoreIDs="h5dict",
                 compression="gzip", compression_opts=3):
        
        self.vectors = {
            # chromosomes for each read.
            "chrms1": "int16", "chrms2": "int16",
            #strand to which the read maps
            "strands1": "bool", "strands2": "bool",
            #Start of the read ("ultrasonic" cut site)
            "cuts1": "int32", "cuts2": "int32"}
        
        self.vectors2 = {
            # fragment lengthes
            "fraglens1": "int32", "fraglens2": "int32",
             # fragid as defined in the manual
            "fragids1": "int64", "fragids2": "int64",
            # midpoint of a fragment, determined as "(start+end)/2"
            "mids1": "int32", "mids2": "int32",
            # distance from a cut site to the restriction fragment
            "dists1": "int32", "dists2": "int32",
             # distance between fragments. If -1, different chromosomes.
             # If -2, different arms.
            "distances": "int32",
            }
        self.vectors3 = {
            # absolute ID of a restriction fragment (0,1,2,... numFragments)
            "rfragAbsIdxs1": "int32", "rfragAbsIdxs2": "int32"}
        
        if dictToStoreIDs == "dict":
            self.rfragIDDict = {}
        elif dictToStoreIDs == "h5dict":
            self.rfragIDDict = h5dict()
        else:
            self.rfragIDDict = dictToStoreIDs
            
        self.metadata = {}
        self.tmpDir = tmpFolder
        if not os.path.exists(self.tmpDir):
            os.makedirs(self.tmpDir)

        #-------Initialization of the genome and parameters-----
        self.mode = mode
        self.genome = genome

        self.chromosomeCount = self.genome.chrmCount
        self.fragIDmult = self.genome.fragIDmult  # used for building heatmaps
        self.rFragIDs = self.genome.rfragMidIds
        self.rFragLens = np.concatenate(self.genome.rfragLens)
        self.rFragMids = np.concatenate(self.genome.rfragMids)
        self.rsites = self.genome.rsiteIds
        # to speed up searchsorted we use positive-only numbers
        self.rsitesPositive = self.rsites + 2*self.fragIDmult

        self.maximumMoleculeLength = maximumMoleculeLength
        
        # File to save the data
        self.filename = os.path.abspath(os.path.expanduser(filename))
        # Chunk size for h5dict operation, external sorting, etc.
        self.chunksize = 10000000

        self.h5dict = h5dict(self.filename, mode=mode, in_memory=False)
        self.h5dict.setCompression(compression, compression_opts)
        
        if "chrms1" in list(self.h5dict.keys()):
            self.N = len(self.h5dict.get_dataset("chrms1"))
        if "metadata" in self.h5dict:
            self.metadata = self.h5dict["metadata"]
    
    def parseInputData(self, dictLike, **kwargs):
        
        if not os.path.exists(dictLike):
            raise IOError('File not found: %s' % dictLike)
        
        dictLike = h5dict(dictLike, 'r')
        
        self.chrms1 = dictLike['chrms1']
        self.chrms2 = dictLike['chrms2']
        self.cuts1 = dictLike['cuts1']
        self.cuts2 = dictLike['cuts2']
        self.strands1 = dictLike['strands1']
        self.strands2 = dictLike['strands2']
        
        self.N = len(self.chrms1)
        
        dictLike['misc']['genome']['idx2label']
        self.updateGenome(self.genome,
                          oldGenome = dictLike["misc"]["genome"]["idx2label"])
        
        DSmask = (self.chrms1 >= 0) * (self.chrms2 >= 0)
        self.metadata['100_DoubleUniqueMapped'] = DSmask.sum()
        
        # Discard dangling ends and self-circles
        sameFragMask = self.evaluate("a = (rfragAbsIdxs1 == rfragAbsIdxs2)", 
                                     ["rfragAbsIdxs1", "rfragAbsIdxs2"]) * DSmask
        cutDifs = self.cuts2[sameFragMask] > self.cuts1[sameFragMask]
        s1 = self.strands1[sameFragMask]
        s2 = self.strands2[sameFragMask]
        SSDE = (s1 != s2)
        SS = SSDE * (cutDifs == s2)
        DM = SSDE & np.logical_not(SS)
        SS_N = SS.sum()
        DM_N = DM.sum()
        SSDE_N = SSDE.sum()
        sameFrag_N = sameFragMask.sum()
        self.metadata['120_SameFragmentReads'] = sameFrag_N
        self.metadata['122_SelfLigationReads'] = SS_N
        self.metadata['124_DanglingReads'] = DM_N
        self.metadata['126_UnknownMechanism'] = sameFrag_N - SSDE_N
        mask = DSmask * (~sameFragMask)
        noSameFrag = mask.sum()
        
        del DSmask
        
        ## Dissect dangling ends ...
        Ddists1 = self.fraglens1 - self.dists1
        Ddists2 = self.fraglens2 - self.dists2
        extD1 = Ddists1[sameFragMask][DM]
        extD2 = Ddists2[sameFragMask][DM]
        extLen = self.fraglens1[sameFragMask][DM].astype(float)
        extD = np.r_['0,2', extD1, extD2].min(0)
        
        del Ddists1, Ddists2, extD1, extD2
        
        # distance between sites facing each other
        dist = self.evaluate("a = numexpr.evaluate('- cuts1 * (2 * strands1 -1) - "
                             "cuts2 * (2 * strands2 - 1)')",
                             ["cuts1", "cuts2", "strands1", "strands2"],
                             constants={"numexpr":numexpr})
        extSpace = dist[sameFragMask][DM]
        
        self.h5dict['_DanglingDetials'] = np.column_stack((extLen, extD, extSpace))
        
        del sameFragMask, extLen, extD, extSpace

        readsMolecules = self.evaluate(
            "a = numexpr.evaluate('(chrms1==chrms2) & (strands1!=strands2) & (dist>=0) &"
            " (dist<=maximumMoleculeLength)')",
            internalVariables=["chrms1", "chrms2", "strands1", "strands2"],
            externalVariables={"dist": dist},
            constants={"maximumMoleculeLength": self.maximumMoleculeLength, "numexpr": numexpr})
        
        mask *= (readsMolecules==False)
        extraDE = mask.sum()
        self.metadata['210_ExtraDanglingReads'] = -extraDE + noSameFrag
        if mask.sum() == 0:
            raise Exception('No reads left after filtering. Please check the input data')

        del dist, readsMolecules
        
        self.maskFilter(mask)
    
    def updateGenome(self, newGenome, oldGenome='current'):

        assert isinstance(newGenome, myGenome)
        
        newN = newGenome.chrmCount
        if oldGenome == "current":
            oldGenome = self.genome
        upgrade = newGenome.upgradeMatrix(oldGenome)
        if isinstance(oldGenome, myGenome):
            if (oldGenome.hasEnzyme()==True) and (newGenome.hasEnzyme()==False):
                newGenome.setEnzyme(oldGenome.enzymeName)
        
        chrms1 = np.array(self.chrms1, int)
        chrms2 = np.array(self.chrms2, int)

        if upgrade is not None:
            upgrade[upgrade == -1] = 9999  # to tell old SS reads from new SS reads

            chrms1 = upgrade[chrms1]
            self.chrms1 = chrms1
            del chrms1

            chrms2 = upgrade[chrms2]
            self.chrms2 = chrms2
            del chrms2
            
        mask = ((self.chrms1 < newN) * (self.chrms2 < newN))
        self.genome = newGenome
        self.maskFilter(mask)
        
    def make_tempfile(self):
        
        tl = time.strftime('%Y%m%d%H%M%S', time.localtime(time.time()))
        kw = {'suffix':tl, 'dir':self.tmpDir}
        fd, tmpfil = tempfile.mkstemp(**kw)
        os.close(fd)
        
        return tmpfil
        
    def filterDuplicates(self, chunkSize=50000000):
        
        if self.N > 200000000:
            mode = "hdd"
        else:
            mode = "ram"
        
        if mode == "ram":
            dups = np.zeros((self.N, 2), dtype="int64", order="C")
            dups[:, 0] = self.chrms1
            dups[:, 0] *= self.fragIDmult
            dups[:, 0] += self.cuts1
            dups[:, 1] = self.chrms2
            dups[:, 1] *= self.fragIDmult
            dups[:, 1] += self.cuts2
            dups.sort(axis=1)
            dups.shape = (self.N * 2)
            strings = dups.view("|S16")
            # Converting two indices to a single string to run unique
            uids = uniqueIndex(strings)
            del strings, dups
            stay = np.zeros(self.N, bool)
            stay[uids] = True  # indexes of unique DS elements
            del uids
        else:
            tmpfil = self.make_tempfile()
            a = h5dict(tmpfil)
            a.add_empty_dataset("duplicates", (self.N,), dtype="|S24")
            a.add_empty_dataset("temp", (self.N,), dtype="|S24")
            dset = a.get_dataset("duplicates")
            tempdset = a.get_dataset("temp")
            code = dedent("""
            tmp = np.array(chrms1, dtype=np.int64) * fragIDmult + cuts1
            tmp2 = np.array(chrms2, dtype=np.int64) * fragIDmult + cuts2
            newarray = np.zeros((len(tmp),3), dtype = np.int64)
            newarray[:,0] = tmp
            newarray[:,1] = tmp2
            newarray[:,:2].sort(axis=1)
            newarray[:,2] = np.arange(start, end, dtype=np.int64)
            newarray.shape = (3*len(tmp))
            a = np.array(newarray.view("|S24"))
            assert len(a) == len(chrms1)
            """)
            self.evaluate(code, ["chrms1", "cuts1", "chrms2", "cuts2"],
                          constants={"np":np, "fragIDmult":self.fragIDmult},
                          outVariable=("a", dset))
            stay = np.zeros(self.N, bool)
            numutils.externalMergeSort(dset, tempdset, chunkSize=chunkSize)
            bins = list(range(0, self.N-1000, self.chunksize)) + [self.N-1]
            for start, end in zip(bins[:-1], bins[1:]):
                curset = dset[start:end+1]
                curset = curset.view(dtype=np.int64)
                curset.shape = (len(curset)//3, 3)
                unique = (curset[:-1,0] != curset[1:,0]) + (curset[:-1,1] != curset[1:,1])
                stay[curset[:,2][unique]] = True
                if end == self.N-1:
                    stay[curset[-1,2]] = True
            del a
            os.remove(tmpfil)
            
        self.metadata["310_DuplicatedRemoved"] = len(stay) - stay.sum()
        self.maskFilter(stay)
    
    def maskFilter(self, mask):
        
        # Uses 16 bytes per read
        for i in list(self.rfragIDDict.keys()):
            del self.rfragIDDict[i]
        
        length = 0
        ms = mask.sum()
        assert mask.dtype == np.bool
        self.N = ms
        if ms == 0:
            return

        for name in self.vectors:
            data = self._getData(name)
            ld = len(data)
            if length == 0:
                length = ld
            else:
                if ld != length:
                    self.delete()
            newdata = data[mask]
            del data
            self._setData(name, newdata)
            del newdata
        del mask
        
        self.h5dict['metadata'] = self.metadata
    
    def _sortData(self):

        if not hasattr(self, "dataSorted"):
            tmpfil = self.make_tempfile()
            mydict = h5dict(tmpfil, 'w')
            data = mydict.add_empty_dataset("sortedData", (self.N,), mydtype)
            tmp = mydict.add_empty_dataset("trash", (self.N,), mydtype)
            code = dedent("""
            a = np.empty(len(chrms1), dtype = mydtype)
            mask = (chrms1 > chrms2) | ( (chrms1 == chrms2) & (cuts1 > cuts2))

            chrms2[mask],chrms1[mask] = chrms1[mask].copy(), chrms2[mask].copy()
            cuts1[mask],cuts2[mask] = cuts2[mask].copy(), cuts1[mask].copy()
            strands1[mask],strands2[mask] = strands2[mask].copy(),strands1[mask].copy()

            a["chrms1"] = chrms1
            a["pos1"] = cuts1
            a["chrms2"] = chrms2
            a["pos2"] = cuts2
            a["strands1"] = strands1
            a["strands2"] = strands2
            """)
            self.evaluate(expression=code, internalVariables=["chrms1","chrms2","cuts1","cuts2","strands1","strands2"],
                          constants={"np":np, "mydtype":mydtype}, outVariable=("a", data))

            externalMergeSort(data, tmp, sorter=mydtypeSorter, searchsorted=searchsorted,
                              chunkSize=max(150000000, self.chunksize))
            sdata = mydict.get_dataset("sortedData")

            c1 = self.h5dict.get_dataset("chrms1")
            c2 = self.h5dict.get_dataset("chrms2")
            p1 = self.h5dict.get_dataset("cuts1")
            p2 = self.h5dict.get_dataset("cuts2")
            s1 = self.h5dict.get_dataset("strands1")
            s2 = self.h5dict.get_dataset("strands2")

            for start, end in self._getChunks():
                data = sdata[start:end]
                c1[start:end] = data["chrms1"]
                c2[start:end] = data["chrms2"]
                p1[start:end] = data["pos1"]
                p2[start:end] = data["pos2"]
                s1[start:end] = data["strands1"]
                s2[start:end] = data["strands2"]
            self.dataSorted = True
            del mydict
            os.remove(tmpfil)
            gc.collect()
    
    def _getChunks(self, chunkSize="default"):
        
        if chunkSize == "default":
            chunkSize = self.chunksize
        if chunkSize > 0.5 * self.N:
            return [(0, self.N)]
        points = list(range(0,self.N-chunkSize//2,chunkSize)) + [self.N]
        
        return list(zip(points[:-1], points[1:]))
    
    def merge(self, filenames):

        h5dicts = [h5dict(i, mode='r') for i in filenames]
        
        if all(["metadata" in i for i in h5dicts]):
            metadatas = [mydict["metadata"] for mydict in h5dicts]
            # print metadatas
            newMetadata = metadatas.pop()
            for oldData in metadatas:
                for key, value in oldData.items():
                    if (key in newMetadata):
                        newMetadata[key] += value
                    else:
                        log.warning('The key %s can not be found in some files',
                                    key)
            self.metadata = newMetadata
            self.h5dict["metadata"] = self.metadata
        
        self.N = sum([len(i.get_dataset("strands1")) for i in h5dicts])

        for name in self.vectors.keys():
            if name in self.h5dict:
                del self.h5dict[name]
            self.h5dict.add_empty_dataset(name, (self.N,), self.vectors[name])
            dset = self.h5dict.get_dataset(name)
            position = 0
            for mydict in h5dicts:
                cur = mydict[name]
                dset[position:position+len(cur)] = cur
                position += len(cur)
            self.h5dict.flush()
            time.sleep(0.2)  # allow buffers to flush
        
        self._sortData()
        
        check = all([('_DanglingDetials' in h) for h in h5dicts])
        if check:
            pool = []
            for mydict in h5dicts:
                tmp = mydict['_DanglingDetials']
                if type(tmp) == dict: # Backward compatibility
                    tmp = np.column_stack((tmp['extLen'],tmp['extD'],tmp['extSpace']))
                pool.append(tmp)
            pool = np.vstack(pool)
            self.h5dict['_DanglingDetials'] = pool
        
        Types = ['LeftType', 'RightType', 'InnerType', 'OuterType']
        pool = {}
        check = all([('_DirectionTypeStats' in h) for h in h5dicts])
        if check:
            for Type in Types:
                tmp = np.zeros(50, dtype=int)
                for mydict in h5dicts:
                    tmp += mydict['_DirectionTypeStats'][Type]
                pool[Type] = tmp
            self.h5dict['_DirectionTypeStats'] = pool
        
        tempDict = {}
        for mydict in h5dicts:
            if 'genomeInformation' in mydict:
                tempDict.update(mydict['genomeInformation'])
        self.h5dict['genomeInformation'] = tempDict
        
    
    def printMetadata(self, saveTo):
        
        self.metadata = self.h5dict['metadata']
        
        longrange = self.metadata['412_IntraLongRangeReads(>=20Kb)']
        contacts = self.metadata['400_TotalContacts']
        longRatio = float(longrange) / contacts
                         
        fromMapping = ['000_SequencedReads', '010_UniqueMappedReads',
                       '020_LigationCounts']
        
        check = all([m in self.metadata for m in fromMapping])
        if check:
            Total = self.metadata['000_SequencedReads']
            Ureads = self.metadata['100_DoubleUniqueMapped']
            ligSeq = self.metadata['020_LigationCounts']
            selfLig = self.metadata['122_SelfLigationReads']
            dangling = self.metadata['124_DanglingReads']
            Uratio = float(Ureads) / Total
            Lratio = float(ligSeq) / Total
            Fratio = float(selfLig) / Total
            Dratio = float(dangling) / Total
            usage = float(contacts) / Total
        ##-----------------------------------------------------------------
        with open(saveTo, 'w') as myfile:
            for i in sorted(self.metadata):
                if (i[2] != '0'):
                    myfile.write('\t\t')
                elif (i[1] != '0') and (i[2] == '0'):
                    myfile.write('\t')
                myfile.write(str(i))
                myfile.write(':   ')
                myfile.write(str(self.metadata[i]))
                myfile.write('\n')
            myfile.write('\nCritical Indicators:\n')
            if check:
                myfile.write('Double Unique Mapped Ratio = %d / %d = %.4f\n' % (Ureads, Total, Uratio))
                myfile.write('Ligation-Junction Ratio = %d / %d = %.4f\n' % (ligSeq, Total, Lratio))
                myfile.write('Self-Ligation Ratio = %d / %d = %.4f\n' % (selfLig, Total, Fratio))
                myfile.write('Dangling-Reads Ratio = %d / %d = %.4f\n' % (dangling, Total, Dratio))
            myfile.write('Long-Range Ratio = %d / %d = %.4f\n' % (longrange, contacts, longRatio))
            if check:
                myfile.write('Data Usage = %d / %d = %.4f\n' % (contacts, Total, usage))
                
    def saveHeatmap(self, filename, resolution, gInfo):

        try:
            os.remove(filename)
        except:
            pass

        tosave = h5dict(path=filename, mode='w')
        
        heatmap = self.buildAllHeatmap(resolution)

        tosave['heatmap'] = heatmap
        
        del heatmap
        
        chromosomeStarts = np.array(self.genome.chrmStartsBinCont)
            
        tosave['resolution'] = resolution
        tosave['chromosomeStarts'] = chromosomeStarts
        tosave['genomeInformation'] = gInfo
    
    def saveByChromosomeHeatmap(self, filename, resolution, gInfo,
                                includeTrans=False):
        
        self.genome.setResolution(resolution)

        mydict = h5dict(filename)

        for chrom in range(self.genome.chrmCount):
            c1 = self.h5dict.get_dataset("chrms1")
            p1 = self.h5dict.get_dataset("cuts1")
            low = h5dictBinarySearch(c1, p1, (chrom,-1), "left")
            high = h5dictBinarySearch(c1, p1, (chrom,999999999), "right")

            chr1 = self._getVector("chrms1", low, high)
            chr2 = self._getVector("chrms2", low, high)
            pos1 = np.array(self._getVector("mids1",low,high)//resolution,
                            dtype=np.int32)
            pos2 = np.array(self._getVector("mids2",low,high)//resolution,
                            dtype=np.int32)

            assert (chr1==chrom).all()  # getting sure that bincount worked

            args = np.argsort(chr2)
            chr2 = chr2[args]
            pos1 = pos1[args]
            pos2 = pos2[args]

            for chrom2 in range(chrom, self.genome.chrmCount):
                if (includeTrans == False) and (chrom2 != chrom):
                    continue
                start = np.searchsorted(chr2, chrom2, "left")
                end = np.searchsorted(chr2, chrom2, "right")
                cur1 = pos1[start:end]
                cur2 = pos2[start:end]
                label = np.array(cur1, "int64")
                label *= self.genome.chrmLensBin[chrom2]
                label += cur2
                maxLabel = self.genome.chrmLensBin[chrom] * \
                           self.genome.chrmLensBin[chrom2]
                counts = np.bincount(label, minlength=maxLabel)
                mymap = counts.reshape((self.genome.chrmLensBin[chrom], -1))
                if chrom==chrom2:
                    mymap = mymap + mymap.T
                    fillDiagonal(mymap, np.diag(mymap).copy()/2)
                mydict["%d %d" % (chrom, chrom2)] = mymap
        
        mydict['resolution'] = resolution
        mydict['genomeInformation'] = gInfo

        return
    
    def buildAllHeatmap(self, resolution):
        
        for start, end in self._getChunks(30000000):
            # 8 bytes per record + heatmap
            self.genome.setResolution(resolution)
            numBins = self.genome.numBins
            label = self.genome.chrmStartsBinCont[self._getVector("chrms1",start,end)]
            label = np.asarray(label,dtype="int64")
            label += (self._getVector("mids1",start,end)//resolution).astype(np.int64)
            label *= numBins
            label += self.genome.chrmStartsBinCont[self._getVector("chrms2",start,end)]
            label += (self._getVector("mids2",start,end)//resolution).astype(np.int64)
            counts = np.bincount(label, minlength=numBins**2)
            if len(counts) > numBins ** 2:
                raise StandardError("\nHeatMap exceed length of the genome!")

            counts.shape = (numBins, numBins)
            try:
                heatmap += counts  # @UndefinedVariable
            except:
                heatmap = counts

        for i in range(len(heatmap)):
            heatmap[i,i:] += heatmap[i:,i]
            heatmap[i:,i] = heatmap[i,i:]
        diag = np.diag(heatmap)
        fillDiagonal(heatmap, diag/2)
        
        return heatmap
    
    def typePlot(self, outfile, dpi = 500):
        
        LT = self.h5dict['_DirectionTypeStats']['LeftType'][:25]
        RT = self.h5dict['_DirectionTypeStats']['RightType'][:25]
        IT = self.h5dict['_DirectionTypeStats']['InnerType'][:25]
        OT = self.h5dict['_DirectionTypeStats']['OuterType'][:25]
        
        Total = LT + RT + IT + OT
        Total = Total.astype(np.float)
        
        LR = LT / Total
        RR = RT / Total
        IR = IT / Total
        OR = OT / Total
        
        fig = plt.figure(figsize = (15, 9))
        ax = fig.add_subplot(111)
        lines = []
        labels = ['Left Type', 'Right Type', 'Inner Type', 'Outer Type']
        x = np.arange(1, LR.size + 1)
        idx = 0
        for y in [LR, RR, IR, OR]:
            L = ax.plot(x, y, color = colorPool[idx], linewidth = 2)
            lines.extend(L)
            idx += 1
        
        ax.set_xlabel('Genomic Separation, KB')
        ax.set_ylabel('Type Ratio')
        
        ax.set_ylim((0, 1))
        ax.set_title('Read Pair Type Statistics')
        ax.xaxis.set_ticks_position('bottom')
        ax.yaxis.set_ticks_position('left')
        
        ax.legend(lines, labels, frameon = False, fontsize = 11, labelspacing = 1,
                  handletextpad = 1, borderpad = 1, markerscale = 1, numpoints = 1,
                  ncol = 2, loc = 'upper right')
        
        plt.savefig(outfile, dpi = dpi)
        plt.close()
    
    def dangStats(self, prefix, offset = 2, dpi = 500):
        
        extD = self.h5dict['_DanglingDetials'][:,1]
        extLen = self.h5dict['_DanglingDetials'][:,0]
        extSpace = self.h5dict['_DanglingDetials'][:,2]
        
        m = int(np.percentile(extD[extD<0],0.5))
        extLen = extLen[extD>=m]
        extSpace = extSpace[extD>=m]
        extD = extD[extD>=m]
        if offset < -m:
            offset = -m
        
        extD += offset
        extR = extD / extLen
        
        lenlow = np.percentile(extLen, 0.1)
        lenhigh = np.percentile(extLen, 99.9)
        spacelow = np.percentile(extSpace, 0.1)
        spacehigh = np.percentile(extSpace, 99.9)
        
        lenM = (extLen >= lenlow) & (extLen <= lenhigh)
        spaceM = (extSpace >= spacelow) & (extSpace <= spacehigh)
        
        fig = plt.figure(figsize = (15, 9))
        ax = fig.add_subplot(111)
        ax.hist(extR[lenM], bins = 15, color = colorPool[-2])
        ax.set_title('Start Site of Dandling Ends Relative to Fragments (Ratio)')
        ax.xaxis.set_ticks_position('bottom')
        ax.yaxis.set_ticks_position('left')
        plt.savefig('-'.join([prefix, 'danglingStart']) + '.png', dpi = dpi)
        plt.close()
        
        fig = plt.figure(figsize = (15, 9))
        ax = fig.add_subplot(111)
        ax.hist(extSpace[spaceM], bins = 15, color = colorPool[-1])
        ax.set_title('Estimated Library Size Distribution (bp)')
        ax.xaxis.set_ticks_position('bottom')
        ax.yaxis.set_ticks_position('left')
        plt.savefig('-'.join([prefix, 'librarySize']) + '.png', dpi = dpi)
        plt.close()

class cBinnedData(binnedData):
    
    def export(self, name, outFilename):

        if not name in self.dataDict:
            raise ValueError("No data {name}".format(name=name))
            
        toexport = {}
        toexport["heatmap"] = self.dataDict[name]
        toexport["resolution"] = self.resolution
        toexport["chromosomeStarts"] = self.chromosomeStarts
        myh5dict = h5dict(outFilename, mode="w")
        myh5dict.update(toexport)

class HiResHiC(object):
    
    def __init__(self, genome, resolution, raw):
        
        self.genome = genome
        self.resolution = resolution
        self.rawdata = h5dict(raw, 'r')
        self.cisKeys = ['{0} {0}'.format(i) for i in self.genome.idx2label]
    
    def iterativeCorrection(self, outname):
        
        mydict = h5dict(outname)
        for key in self.cisKeys:
            bychr = self.rawdata[key]
            corrected = completeIC(bychr, returnBias=False)
            mydict[key] = corrected
        mydict["resolution"] = self.resolution


# Created on Wed Dec 24 15:26:26 2014

# Author: XiaoTao Wang
# Organization: HuaZhong Agricultural University


import os, sys, lib
from distutils.core import setup

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

if (sys.version_info.major != 2) or (sys.version_info.minor < 6):
    print 'PYTHON VERSION MUST BE 2.6 or 2.7. YOU ARE CURRENTLY USING PYTHON ' + sys.version
    sys.exit(2)

# Guarantee Unix Format
text = open('scripts/runHiC', 'rb').read().replace('\r\n', '\n')
open('scripts/runHiC', 'wb').write(text)
text = open('scripts/runHiC-mapping', 'rb').read().replace('\r\n', '\n')
open('scripts/runHiC-mapping', 'wb').write(text)

setup(
    name = 'runHiC',
    version = lib.__version__,
    author = lib.__author__,
    author_email = 'wangxiaotao868@163.com',
    url = 'https://github.com/XiaoTaoWang/HiC_pipeline',
    description = 'A easy-to-use Hi-C processing software based on hiclib',
    keywords = 'Hi-C HiC ICE Contact',
    package_dir = {'runHiC':'lib'},
    packages = ['runHiC'],
    scripts = ['scripts/runHiC','scripts/runHiC-mapping'],
    long_description = read('README.rst'),
    classifiers = [
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Operating System :: POSIX',
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        ]
    )
